# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006-2007 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


package Wanha::Chan;

use strict;
use warnings FATAL => 'all';
use diagnostics;

use Wanha::DB;


my %name_to_id;
my %id_to_name;


=item add
Add channel TODO join

0: chan name
r: chan id
=cut
sub add($$) {
	my ($name) = @_;
	my $id;
	my $update;

	&Wanha::DB::transaction_start();
	$id = &Wanha::DB::get_sequence("chans_id_seq");
	$update = &Wanha::DB::prepare(
			"INSERT INTO chans (id, chan) " .
			"VALUES (&da&, &sb&)");
	$update = &Wanha::DB::set($update,
			"a", $id,
			"b", $name);
	&Wanha::DB::update($update);
	&Wanha::DB::transaction_end();

	$name_to_id{$name} = $id;
	$id_to_name{$id} = $name;

	return $id;
}


=name get_id
Get channel id (by name)

0: chan name
1: auto-add (define to enable)
r: chan id
=cut
sub get_id($$) {
	my ($name, $autoadd) = @_;
	my ($query, $result);

	# Try from cache
	(defined($name_to_id{$name})) && return $name_to_id{$name};

	$query = &Wanha::DB::prepare(
			"SELECT id, chan FROM chans " .
			"WHERE chan ILIKE &sa&");
	$query = &Wanha::DB::set($query, "a", $name);
	$result = &Wanha::DB::query($query);

	if (&Wanha::DB::row_count($result) == 1) {
		my ($id, $t) = &Wanha::DB::get_row($result);
		$name_to_id{$name} = $id;
		return $id;
	} elsif (&Wanha::DB::row_count($result) == 0) {
		if (defined($autoadd) && $autoadd == 1) {
			return &add($name);
		}
		return undef;
	}

	# more than one entry? PostgreSQL/ILIKE seems to handle '_' as a
	# wildcard... Fine, we'll manually pick the correct one.
	$name =~ tr/A-Z/a-z/;
	while (my ($id, $str) = &Wanha::DB::get_row($result)) {
		$str =~ tr/A-Z/a-z/;
		if ($str eq $name) {
			$name_to_id{$name} = $id;
			return $id;
		}
	}
	return undef;
}


=item get_id_smart
Get channel id either by name or ID

0: chan id / chan name
r: chan id
=cut
sub get_id_smart($) {
	my ($name) = @_;
	my $id;

	if ($name eq &Wanha::Lang::get("txt-any")) {
		return -1;
	} elsif (! $name =~ /./) {
		return undef;
	} elsif ($name =~ /^[0-9]/) {
		return $name;
	} else {
		return &get_id($name);
	}
}


=item get_name
Get channel name (by ID)

0: id
r: chan
=cut
sub get_name($) {
	my ($id) = @_;
	my ($query, $result);

	# Try from cache
	(defined($id_to_name{$id})) && return $id_to_name{$id};

	$query = &Wanha::DB::prepare("SELECT chan FROM chans WHERE id = &da&");
	$query = &Wanha::DB::set($query, "a", $id);
	$result = &Wanha::DB::query($query);
	(&Wanha::DB::row_count($result) == 0) && return undef;

	my ($name) = &Wanha::DB::get_row($result);
	$id_to_name{$id} = $name;

	return $name;
}


return 1;
